
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  03-action.t - test script for Action Modules.
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

use Test;
use File::stat;
use File::Compare;
use File::Basename;
use File::Path;
use PSGConf::Util;

use lib './t';
use t;

File::Find::find( { wanted => \&t::wanted }, 'lib/PSGConf/Action' );

# There are going to be 3 tests for each module, new(), check(), and do()
# diff() only prints out to the screen, so we will forget about that for now.
plan tests => scalar @t::found * 3;

$name    = $ENV{'PSGCONF_TMPDIR'} . '/test';
$psgconf = PSGConf->new(%t::psgconf_options);

# The gid might be a space delimited list of gids, so just
# take the first one.
@gids = split /\s+/, $(;

# Special options to each Action module
$options = {
    'PSGConf::Action::ChGrp'    => { gid  => $gids[0] },
    'PSGConf::Action::ChMod'    => { mode => 0511 },
    'PSGConf::Action::ChOwn'    => { uid  => $< },
    'PSGConf::Action::CopyFile' => {
        copy_from => 'magic',
        uid       => $<,
        gid       => $gids[0]
    },
    'PSGConf::Action::CreateFile' => {
        mode => 0600,
        uid  => $<,
        gid  => $gids[0]
    },
    'PSGConf::Action::Crontab' => {
        user    => scalar( getpwuid($<) ),
        uid     => $<,
        entries => [
            {
                hour    => 0,
                command => '/bin/true'
            }
        ]
    },
    'PSGConf::Action::GenerateFile' => {
        uid      => $<,
        gid      => $gids[0],
        generate => sub { return 1; }
    },
    'PSGConf::Action::HomeDir' => {
        user_info => {
            populate_method => sub {
                create_file( $name . '/.profile',
                    "PATH=/usr/bin:/usr/local/bin" );
              }
        },
        user => scalar( getpwuid($<) ),
        type => 'd',                      # In here for check_file routine usage
        uid  => $<,
        gid  => $gids[0],
        mode => 0700,
    },
    'PSGConf::Action::MkDir' => {
        type => 'd',                      # In here for check_file routine usage
        mode => 0700,
        uid  => $<,
        gid  => $gids[0]
    },
    'PSGConf::Action::MkNod' => {
        type  => 'b',
        mode  => 0600,
        major => 1,
        minor => 2,
        uid   => $<,
        gid   => $gids[0]
    },
    'PSGConf::Action::ModifyFile' => {
        modify_func => sub {
            my ( $action, $infh, $fh, $psgconf ) = @_;
            my ($line);

            while ( $line = <$infh> )
            {
                $line = "line 2\n"
                  if ( $line =~ /^this line needs to be changed$/ );
                print $fh $line;
            }
            return 1;
        },
        type => 'f',       # In here for check_file routine usage
        mode => 0600,
        uid  => $<,
        gid  => $gids[0]
    },
    'PSGConf::Action::PackageManager'             => { seperator => ['-'] },
    'PSGConf::Action::PackageManager::Encap'      => { seperator => ['-'] },
    'PSGConf::Action::PackageManager::FTPArchive' => { seperator => ['-'] },
    'PSGConf::Action::PackageManager::FreeBSD' =>
      { seperator => [ '-', '_', ',' ] },
    'PSGConf::Action::PackageManager::FreeBSD::Packages' =>
      { seperator => [ '-', '_', ',' ] },
    'PSGConf::Action::PackageManager::FreeBSD::Ports' =>
      { seperator => [ '-', '_', ',' ] },
    'PSGConf::Action::PackageManager::Redhat::RHN'  => { seperator => ['-'] },
    'PSGConf::Action::PackageManager::Redhat::RPMS' => { seperator => ['-'] },
    'PSGConf::Action::PackageManager::RedHat::Yum'  => { seperator => ['-'] },
    'PSGConf::Action::PackageManager::Solaris'      => { seperator => ['-'] },
    'PSGConf::Action::RPMImportGPGKey'              => {
        rpm_cmd   => '/bin/rpm',
        key       => '4e23daca',
        keyserver => 'hkp://subkeys.pgp.net'
    },
    'PSGConf::Action::RemoveCrontab' =>
      { fullpath => '/var/spool/cron/' . getpwuid($<) },
    'PSGConf::Action::RestartDaemon' => {
        check_func => \&RestartDaemon_check_func,
        pidfile    => [ $name . '.pid' ]
    },
    'PSGConf::Action::RunCommand' => {
        check_func => \&RestartDaemon_check_func,
        command    => 'touch ' . $name
    },
    'PSGConf::Action::Symlink' => {
        link_to => $name . '.orig',
        backup  => 0
    },
    'PSGConf::Action::TouchFile' => {
        uid       => $<,
        gid       => $gids[0],
        timestamp => 100
    },
    'PSGConf::Action::UntarFile' => {
        user    => scalar( getpwuid($<) ),
        tar_cmd => 'tar',
        tarball => 'test.tar'
    },
    'PSGConf::Action::lpadmin' => {
        default_printer => $name,
        device          => '/dev/null',
        model           => '/usr/lib/lp/model/netstandard',
        protocol        => 'tcp',
        content_type    => [ 'simple', 'postscript', 'pcl' ]
    },
    'PSGConf::Action::GenerateFile::logadm' => {
        count => 14,
        files => [ "/tmp/foo", "/tmp/bar" ]
    },
    'PSGConf::Action::GenerateFile::logrotate' => {
        count => 14,
        files => [ "/tmp/foo", "/tmp/bar" ]
    },
    'PSGConf::Action::GenerateFile::newsyslog' => {
        count => 14,
        files => [ "/tmp/foo", "/tmp/bar" ]
    }
};

foreach my $module (@t::found)
{

    ###
    ### Massage the file name to convert it into perl use syntax
    ###
    $module =~ s,^lib/,,;
    $module =~ s,/,::,g;
    $module =~ s/.pm$//;

    eval "use $module;";

    $test = 'test_' . $module;
    $test =~ s/test_.*::/test_/;

    if ( $module =~ /PackageManager::RedHat/ && $^O ne 'linux' )
    {
        skip(
"Skip (modules that require RPM perl module only tested on linux systems)",
            0, 0, "$module->new() failed"
        );
        skip(
"Skip (modules that require RPM perl module only tested on linux systems)",
            0, 0, "$module->check() failed"
        );
        skip(
"Skip (modules that require RPM perl module only tested on linux systems)",
            0, 0, "$module->do() failed"
        );
        next;
    }

    # printf STDERR "Testing $module\n";

    $action = create_action($module);
    if ( defined &{$test} )
    {
        &{$test}( $module, $psgconf );
    } elsif ( defined $action )
    {
        $action->conf($psgconf)
          if ( $action->can('conf') );
        $action->check($psgconf);

        #ok ($action->changed(), 1, "$module->check() failed" );

        # skip because of no custom testing
        skip( "Skip (no custom test case $module->check())",
            0, 0, "$module->check() failed" );
        skip( "Skip (no custom test case $module->do())",
            0, 0, "$module->do() failed" );
    } else
    {
        ok( 0, 1, "$module->check() failed" );
        ok( 0, 1, "$module->do() failed" );
    }

    $action = undef;
}

sub create_action
{
    my ($module) = @_;
    my ($action);

    if ( $module->can('new') )
    {
        $action = $module->new(
            name => $name,
            %{ $options->{$module} }
        );
    }

    ok( defined $action, 1, "$module->new() failed" );

    return $action;
}

sub create_file
{
    my ( $name, $content ) = @_;
    local *MYFP;

    # Create the file so we can test against it later
    open( MYFP, ">$name" ) || die "Could not create file ($name) $!\n";
    printf MYFP $content if ( defined $content );
    close MYFP;
}

sub check_file
{
    my ( $module, $name ) = @_;
    my ( $res, $msg );

    # now check that the file was created properly
    if ( -e $name )
    {
        my $st = stat($name);
        if ( ( $st->mode & 07777 ) == $options->{$module}->{mode} )
        {
            if ( $st->uid == $options->{$module}->{uid} )
            {
                if ( $st->gid == $options->{$module}->{gid} )
                {
                    if (
                        ( $options->{$module}->{type} eq 'b' && -b $name )
                        || ( $options->{$module}->{type} eq 'c'
                            && -c $name )
                        || ( $options->{$module}->{type} eq 'd'
                            && -d $name )
                        || ( $options->{$module}->{type} eq 'f'
                            && -f $name )
                      )
                    {
                        $res = 1;
                    } else
                    {
                        $res = 0;
                        $msg = "$module->do() failed (type not correct)";
                    }
                } else
                {
                    $res = 0;
                    $msg = "$module->do() failed (gid not correct)";
                }
            } else
            {
                $res = 0;
                $msg = "$module->do() failed (uid not correct)";
            }
        } else
        {
            $res = 0;
            $msg = "$module->do() failed (mode not correct)";
        }
    } else
    {
        $res = 0;
        $msg = "$module->do() failed (file does not exist)";
    }

    return ( $res, $msg );
}

################################################################################
### Action module specific tests
################################################################################
sub test_ChGrp
{
    my ( $module, $psgconf ) = @_;

    create_file($name);

    # Change the group to a different group we are a member of
    if ( scalar @gids < 3 )
    {
        skip( "Skip (not members of enough groups", 0, 0, 0 );
        skip( "Skip (not members of enough groups", 0, 0, 0 );
        return;
    }

    chown( -1, $gids[2], $name );

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );

    $action->do($psgconf)
      if ( $action->changed() );

    my $st = stat($name);
    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        $st->gid == $gids[0],
        1, "$module->do() failed"
    );

    unlink($name) if ( -f $name );
}

sub test_ChMod
{
    my ( $module, $psgconf ) = @_;

    create_file($name);

    # Change the mode to make sure we change it back correctly
    chmod( 0777, $name );

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );

    $action->do($psgconf)
      if ( $action->changed() );

    my $st = stat($name);
    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        ( $st->mode & 07777 ) == $options->{'PSGConf::Action::ChMod'}->{mode},
        1,
        "$module->do() failed"
    );

    unlink($name) if ( -f $name );
}

sub test_ChOwn
{
    my ( $module, $psgconf ) = @_;

    # If we are running the test as root, then chown the file
    # away from root to then test the do() subroutine.
    #
    # NOTE: Cannot use the code as listed in the chown() perldoc:
    #	use POSIX qw(sysconf _PC_CHOWN_RESTRICTED);
    #	$can_chown_giveaway = not sysconf(_PC_CHOWN_RESTRICTED);
    # because in the do() routine, we would be chowning back to us,
    # and that is not allowed either.
    if ( $< == 0 )
    {
        create_file($name);
        chown( 1, -1, $name );
        $action->check($psgconf);
        skip( !defined $action ? "Skip (failed $module->new())" : 0,
            $action->changed(), 1, "$module->check() failed" );

        $action->do($psgconf)
          if ( $action->changed() );

        my $st = stat($name);
        skip(
            !$action->changed() ? "Skip (failed $module->check())" : 0,
            $st->uid == $<,
            1, "$module->do() failed"
        );
    } else
    {
        skip( "Skip (non root)", 1, "$module->check() failed" );
        skip( "Skip (non root)", 1, "$module->do() failed" );
    }

    unlink($name) if ( -f $name );
}

sub test_CopyFile
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );

    $action->do($psgconf)
      if ( $action->changed() );
    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        compare(
            $name,
            $psgconf->{files_dir} . '/'
              . $options->{'PSGConf::Action::CopyFile'}->{'copy_from'}
        ),
        0,
        "$module->do() failed"
    );

    unlink($name) if ( -f $name );
}

sub test_CreateFile
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );

    $action->do($psgconf)
      if ( $action->changed() );
    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        -f $name, 1, "$module->do() failed" );

    # Clean up after ourselves
    unlink($name) if ( -f $name );
}

sub test_Crontab
{
    my ( $module, $psgconf ) = @_;
    my ($savefile) = 'crontab.orig';

    # Save the existing crontab file if this user already has one
    PSGConf::Util::RunCommand( "crontab -l > $savefile 2> /dev/null", 1 );

    # If there was no crontab entry, then remove the file
    unlink($savefile) if ( -z $savefile );

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );

    # Now grep to see if the /usr/bin/true command is really in the crontab file
    local *FP;
    open( FP, 'crontab -l |' );
    my $crontab = grep (
        /$options->{'PSGConf::Action::Crontab'}->{'entries'}->[0]->{'command'}/,
        <FP> );
    close FP;

    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        $crontab, 1, "$module->do() failed" );

    # restore the original crontab file
    if ( -f $savefile )
    {
        PSGConf::Util::RunCommand("crontab < $savefile");
        unlink($savefile);
    } else
    {
        PSGConf::Util::RunCommand("crontab -r");
    }
}

sub test_File
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip(
        !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(),
        0,    # Action::File never returns 1
        "$module->check() failed"
    );

    # $action->do($psgconf);
    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        0, 1, "$module->do() failed" );
}

sub test_GenerateFile
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );

    $action->do($psgconf)
      if ( $action->changed() );

    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        -f $name, 1, "$module->do() failed" );

    unlink($name) if ( -f $name );
}

sub test_HomeDir
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );

    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        -f $name . '/.profile',
        1, "$module->do() failed"
    );

    # Clean up after ourselves
    unlink( $name . '/.profile' ) if ( -f $name . '/.profile' );
    rmdir($name) if ( -d $name );
}

sub test_MkDir
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );

    my ( $res, $msg ) = check_file( $module, $name );
    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        $res, 1, $msg );

    # Clean up after ourselves
    rmdir($name) if ( -d $name );
}

sub test_MkNod
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );

    # If we are not root, then mknod will not work
    if ( $< == 0 )
    {
        $action->do($psgconf)
          if ( $action->changed() );

        my ( $res, $msg ) = check_file( $module, $name );
        skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
            $res, 1, $msg );
    } else
    {
        skip( "Skip (non root)", 1, "$module->do() failed" );
    }

    # Clean up after ourselves
    rmdir($name) if ( -d $name );
}

sub test_ModifyFile
{
    my ( $module, $psgconf ) = @_;
    my ($cnt);
    local *FP;

    create_file( $name,
        "line\nline 1\nthis line needs to be changed\nline 3\n" );

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );

    $action->do($psgconf)
      if ( $action->changed() );

    # Now grep to see if the line did get changed
    open( FP, $name );
    $cnt = grep ( /^line \d$/, <FP> );
    close FP;

    my ( $res, $msg ) = check_file( $module, $name );

    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        $cnt, 3, "$module->do() failed" );

    # Clean up after ourselves
    rmdir($name) if ( -d $name );
}

sub test_PackageManager
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip(
        !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(),
        0,    # Action::PackageManager needs an real system
        "$module->check() failed"
    );

    # $action->do($psgconf);
    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        0, 1, "$module->do() failed" );
}

sub test_RPMImportGPGKey
{
    my ( $module, $psgconf ) = @_;

    if ( !-f $options->{'PSGConf::Action::RPMImportGPGKey'}->{'rpm_cmd'} )
    {
        skip( "Skip (no rpm cmd)", 1, "$module->check() failed" );
        skip( "Skip (no rpm cmd)", 1, "$module->do() failed" );
    } elsif ( $< != 0 )
    {
        skip( "Skip (not root)", 1, "$module->check() failed" );
        skip( "Skip (not root)", 1, "$module->do() failed" );
    } else
    {
        $action->check($psgconf);
        skip( !defined $action ? "Skip (failed $module->new())" : 0,
            $action->changed(), 1, "$module->check() failed" );
        $action->do($psgconf)
          if ( $action->changed() );
        skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
            -e $name, undef, "$module->do() failed" );

        # Clean up after ourselves
        system( $options->{'PSGConf::Action::RPMImportGPGKey'}->{'rpm_cmd'}
              . ' -e gpg-pubkey-'
              . $options->{'PSGConf::Action::RPMImportGPGKey'}->{'key'} );
    }
}

sub test_Remove
{
    my ( $module, $psgconf ) = @_;

    create_file($name);

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );
    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        -e $name, undef, "$module->do() failed" );
}

sub test_RemoveCrontab
{
    my ( $module, $psgconf ) = @_;
    my ($savefile) = 'crontab.orig';

    if ( $< != 0 )
    {
        skip( "Skip (not root)", 1, "$module->check() failed" );
        skip( "Skip (not root)", 1, "$module->do() failed" );

    } else
    {

        # Save the existing crontab file if this user already has one
        PSGConf::Util::RunCommand( "crontab -l > $savefile 2> /dev/null", 1 );

        # If there was no crontab entry, then remove the file
        unlink($savefile) if ( -z $savefile );

        ### create the crontab to remove for this user
        `echo "0 * * * * /usr/bin/true" | crontab`;

        $action->check($psgconf);
        skip( !defined $action ? "Skip (failed $module->new())" : 0,
            $action->changed(), 1, "$module->check() failed" );
        $action->do($psgconf)
          if ( $action->changed() );
        skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
            -e $name, undef, "$module->do() failed" );

        # restore the original crontab file
        if ( -f $savefile )
        {
            PSGConf::Util::RunCommand("crontab < $savefile");
            unlink($savefile);
        } else
        {
            my $res =
              PSGConf::Util::RunCommand( "crontab -l > /dev/null 2> /dev/null",
                1 );

            # Clean up if we still have the crontab around.
            PSGConf::Util::RunCommand( "crontab -r > /dev/null 2> /dev/null",
                1 )
              if ( !$res );
        }
    }
}

sub RestartDaemon_check_func
{
    return 1;
}

sub test_RestartDaemon
{
    my ( $module, $psgconf ) = @_;

    # FIXME: There are 4 different ways to restart a daemon.
    # We should test all of them.  They are:
    # 	refresh command (only on AIX)
    #	pidfile
    #	rc startup/shutdown script
    # 	cmdline

    # Create a daemon to 'restart' with a pid file.
    create_file( $name . '.pid' );

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );
    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        -e $name . '.pid',
        1, "$module->do() failed"
    );

    unlink( $name . '.pid' ) if ( -f $name . '.pid' );
}

sub test_RunCommand
{
    my ( $module, $psgconf ) = @_;

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );
    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        -e $name, 1, "$module->do() failed" );

    unlink($name) if ( -f $name );
}

sub test_Symlink
{
    my ( $module, $psgconf ) = @_;

    create_file( $name . '.orig' );

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );
    skip( !$action->changed() ? "Skip (failed $module->check())" : 0,
        -l $name, 1, "$module->do() failed" );

    # Clean up after ourselves
    unlink($name) if ( -l $name );
    unlink( $name . '.orig' ) if ( -f $name . '.orig' );
}

sub test_TouchFile
{
    my ( $module, $psgconf ) = @_;

    create_file($name);

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );
    my $st = stat($name);
    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        $st->mtime == 100,
        1, "$module->do() failed"
    );

    # Clean up after ourselves
    unlink($name) if ( -f $name );
}

sub test_UntarFile
{
    my ( $module, $psgconf ) = @_;

    # Create a tarball, that has file foo in it.
    mkdir($name);
    create_file( $name . '/foo' );
    create_file( $name . '/foo2' );
    `tar -cf $options->{'PSGConf::Action::UntarFile'}->{'tarball'} $name/foo*`;
    unlink( $name . '/foo' );
    unlink( $name . '/foo2' );

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );

    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        -f $name . '/' . $name . '/foo',
        1, "$module->do() failed"
    );
    unlink( $options->{'PSGConf::Action::UntarFile'}->{'tarball'} )
      if ( -f $options->{'PSGConf::Action::UntarFile'}->{'tarball'} );
    rmtree($name) if ( -d $name );
}

sub test_lpadmin
{
    my ( $module, $psgconf ) = @_;

    # FIXME, have not tested this running as root.
    if ( !-f '/usr/bin/lpstat' || !-f '/usr/sbin/lpadmin' || $< != 0 )
    {
        skip( "no lpstat/lpadmin command(s), or not root",
            undef, undef, "$module->check() failed" );
        skip( "no lpstat/lpadmin command(s), or not root",
            undef, undef, "$module->do() failed" );
        return;
    }

    $action->check($psgconf);
    skip( !defined $action ? "Skip (failed $module->new())" : 0,
        $action->changed(), 1, "$module->check() failed" );
    $action->do($psgconf)
      if ( $action->changed() );

    skip(
        !$action->changed() ? "Skip (failed $module->check())" : 0,
        `lpstat -d`,
        "system default destination: $name",
        "$module->do() failed"
    );

    `lpadmin -x $name`;
}
