
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  04-data.t - test script for Data Modules.
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

use Test;

use lib "./t";
use t;

BEGIN { plan tests => 97; }

################################################################################

use PSGConf::Data::Enum;
$enum = PSGConf::Data::Enum->new(
    default => 'ignore',
    values  => [ ['enable'], ['disable'], ['ignore'] ]
);
ok( $enum->get(), undef, "Failed Data::Enum->new()" );

$enum->set();
ok( $enum->get(),            2, "Data::Enum error in set(default)" );
ok( $enum->equals('ignore'), 1, "Data::Enum error in equals()" );

$enum->set('enable');
ok( $enum->get(),             0, "Data::Enum error in set(enable)" );
ok( !$enum->equals('ignore'), 1, "Data::Enum error in equals()" );
ok( $enum->exists(),          1, "Data::Enum error in exists()" );
$enum->unset();
ok( $enum->get(), undef, "Data::Enum error in unset()" );

################################################################################
use PSGConf::Data::Boolean;

$bool = PSGConf::Data::Boolean->new();

ok( $bool->get(), undef, "Failed Data::Boolean->new()" );

$bool->set();
ok( $bool->get(),           1,  "Data::Boolean error in set(default)" );
ok( $bool->equals('false'), "", "Data::Boolean error in equals()" );
ok( $bool->exists(),        1,  "Data::Boolean error in exists" );
$bool->unset();
ok( $bool->get(), undef, "Data::Boolean error in unset()" );

################################################################################
use PSGConf::Data::Integer;

$int = PSGConf::Data::Integer->new();
ok( $int->get(), undef, "Failed Data::Integer->new()" );

$int->set(0xa);
ok( $int->get(), 10, "Data::Integer cannot handle hex values" );

$int->set(010);
ok( $int->get(), 8, "Data::Integer cannot handle octal values" );

ok( $int->exists(), 1, "Data::Integer error in exists" );

$int->add(10);
ok( $int->get(), 18, "Data::Integer error in addition" );

$int->sub(0x2);
ok( $int->get(), 16, "Data::Integer error in subtraction" );

$int->div(010);
ok( $int->get(), 2, "Data::Integer error in division" );

$int->mult(0xC);
ok( $int->get(), 24, "Data::Integer error in multiplication" );

$int->incr();
ok( $int->get(), 25, "Data::Integer error in increment" );

$int->decr();
ok( $int->get(), 24, "Data::Integer error in decrement" );

ok( $int->eq(24), 1,  "Data::Integer error in eq()" );
ok( $int->eq(25), "", "Data::Integer error in eq()" );
ok( $int->ne(24), "", "Data::Integer error in eq()" );
ok( $int->ne(25), 1,  "Data::Integer error in eq()" );
ok( $int->lt(20), "", "Data::Integer error in lt()" );
ok( $int->le(24), 1,  "Data::Integer error in le()" );
ok( $int->gt(20), 1,  "Data::Integer error in gt()" );
ok( $int->ge(24), 1,  "Data::Integer error in ge()" );
$int->unset();
ok( $int->get(), undef, "Data::Integer error in unset()" );

################################################################################
use PSGConf::Data::String;

$string = PSGConf::Data::String->new();
ok( $string->get(), undef, "Failed Data::String->new()" );

$string->set('foo');
ok( $string->get(), 'foo', "Data::String error in set()" );

$string->append(' bar');
ok( $string->get(), 'foo bar', "Data::String error in append()" );

$string->prepend('baz ');
ok( $string->get(), 'baz foo bar', "Data::String error in prepend()" );

$string->gsub( 'ba', 'Ba' );
ok( $string->get(), 'Baz foo Bar', "Data::String error in gsub()" );

ok( $string->match('[bB]a'), 1, "Data::String error in match()" );
$string->unset();
ok( $string->get(), undef, "Data::String error in unset()" );

################################################################################
use PSGConf::Data::List;

$list = PSGConf::Data::List->new();

ok( $list->count(),      0,   "Data::List error in count(0)" );
ok( $list->add('a'),     1,   "Data::List error in add()" );
ok( $list->add_top('1'), 1,   "Data::List error in add_top()" );
ok( $list->count(),      2,   "Data::List error in count(2)" );
ok( $list->find('a'),    'a', "Data::List error in find('a')" );
ok( $list->delete('a'),  1,   "Data::List error in delete()" );
$list->set('z');
ok( $list->count(), 1, "Data::List error in set()" );
$list->unset();
ok( $list->count(), 0, "Data::List error in count(0)" );
$list->set( [ 1, 2, 3 ] );
ok( $list->first(), 1,     "Data::List error in first()" );
ok( $list->next(),  2,     "Data::List error in next()" );
ok( $list->next(),  3,     "Data::List error in next()" );
ok( $list->next(),  undef, "Data::List error in next()" );
ok( $list->first(), 1,     "Data::List error in next()" );

################################################################################
use PSGConf::Data::Hash;

$hash = PSGConf::Data::Hash->new();
ok( $hash->count(), 0, "Data::Hash error in count(0)" );

$hash->set( { 'name' => 'foo' } );
ok( $hash->count(), 1, "Data::Hash error in count(1)" );

$hash->insert( { 'descr' => 'test case' } );
ok( $hash->count(), 2, "Data::Hash error in count(2)" );

ok( $hash->find('name'),    'foo', "Data::Hash error in find()" );
ok( $hash->find('foo'),     undef, "Data::Hash error in find(undef)" );
ok( $hash->delete('descr'), 1,     "Data::Hash error in delete()" );

$hash->unset();
ok( $hash->count(), 0, "Data::Hash error in count(0)" );

$hash->set( { 'one' => 1, 'two' => 2 } );
ok( $hash->first(), 1,     "Data::Hash error in first()" );
ok( $hash->next(),  2,     "Data::Hash error in next()" );
ok( $hash->next(),  undef, "Data::Hash error in next()" );
ok( $hash->first(), 1,     "Data::Hash error in next()" );

################################################################################
use PSGConf::Data::Table;

$table = PSGConf::Data::Table->new();
ok( $table->count_row(), 0, "Data::Table error in count_row(0)" );

$table->set( [ [ 'sshd', 'localhost', 'allow' ] ] );
$table->insert_row( { 0 => '\bsshd\b' }, [ 'ssh_root', 'my.host', 'allow' ] );
ok( $table->count_row(), 2, "Data::Table error in count_row(2)" );
ok( $table->count_col(), 3, "Data::Table error in count_col()" );
ok( defined $table->find_row( { 0 => '\bsshd\b' } ),
    1, "Data::Table error in find_row()" );
$table->replace_row_cells( { 0 => '\bsshd\b' }, { 1 => 'localhost.new' } );
ok( $table->count_row(), 2, "Data::Table error in count_row(2)" );
$table->append_to_row_cells( { 0 => '\bssh_root\b' },
    { 1 => 'localhost.new' } );
ok( $table->count_row(), 2, "Data::Table error in count_row(2)" );
$table->delete_row( { 0 => '\bssh_root\b' } );
ok( $table->count_row(), 1, "Data::Table error in count_row(1)" );

$table->set( [ [ 'sshd', 'localhost', 'allow' ] ] );
ok( $table->first_row()->[0], 'sshd', "Data::Table error in first_row()" );
ok( $table->next_row(),       undef,  "Data::Table error in next_row()" );
ok( $table->first_row()->[0], 'sshd', "Data::Table error in next_row()" );

################################################################################
use PSGConf::Data::FileTest;

$filetest = PSGConf::Data::FileTest->new();

ok( $filetest->read('t/04-data.t'),  1, "Data::FileTest error in read()" );
ok( $filetest->write('t/04-data.t'), 1, "Data::FileTest error in write()" );
ok( $filetest->execute('t/04-data.t'), "",
    "Data::FileTest error in execute()" );
skip(
    $< == 0 ? "Skip (owner test as root)" : undef,
    $filetest->owner('t/04-data.t'),
    1, "Data::FileTest error in owner()"
);
ok( $filetest->exists('t/04-data.t'), 1,  "Data::FileTest error in exists()" );
ok( $filetest->zero('t/04-data.t'),   "", "Data::FileTest error in zero()" );
ok( $filetest->zero('pm_to_blib'),    1,  "Data::FileTest error in zero()" );
ok(
    $filetest->nonzero('t/04-data.t'),
    -s 't/04-data.t',
    "Data::FileTest error in nonzero()"
);
ok( $filetest->nonzero('pm_to_blib'), 0, "Data::FileTest error in zero()" );
ok( $filetest->plain('t/04-data.t'),  1, "Data::FileTest error in plain()" );
ok( $filetest->directory('t/04-data.t'),
    "", "Data::FileTest error in directory()" );
ok( $filetest->directory('t/'), 1, "Data::FileTest error in directory()" );
ok( $filetest->symlink('t/04-data.t'), "",
    "Data::FileTest error in symlink()" );
ok( $filetest->socket('t/04-data.t'), "", "Data::FileTest error in socket()" );
ok( $filetest->fifo('t/04-data.t'),   "", "Data::FileTest error in fifo()" );
ok( $filetest->block('t/04-data.t'),  "", "Data::FileTest error in block()" );
ok( $filetest->character('t/04-data.t'),
    "", "Data::FileTest error in character()" );
ok( $filetest->setuid('t/04-data.t'), "", "Data::FileTest error in setuid()" );
ok( $filetest->sticky('t/04-data.t'), "", "Data::FileTest error in sticky()" );
ok( $filetest->filehandle('t/04-data.t'),
    undef, "Data::FileTest error in filehandle()" );
ok( $filetest->text('t/04-data.t'),   1,  "Data::FileTest error in text()" );
ok( $filetest->binary('t/04-data.t'), "", "Data::FileTest error in binary()" );
ok(
    $filetest->modifytime('t/04-data.t'),
    -M 't/04-data.t',
    "Data::FileTest error in modifytime()"
);
ok(
    $filetest->accesstime('t/04-data.t'),
    -A 't/04-data.t',
    "Data::FileTest error in accesstime()"
);
ok(
    $filetest->inodetime('t/04-data.t'),
    -C 't/04-data.t',
    "Data::FileTest error in inodetime()"
);
