
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  05-datastore.t - test script for DataStore Modules.
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

use Test;

use lib "./t";
use t;

File::Find::find( { wanted => \&t::wanted }, 'lib/PSGConf/DataStore' );

plan tests => scalar @t::found * 2;

foreach $module (@t::found)
{

    ###
    ### Massage the file name to convert it into perl use syntax
    ###
    $module =~ s,^lib/,,;
    $module =~ s,/,::,g;
    $module =~ s/.pm$//;

    if ( $module =~ /::AppConfig$/ )
    {
        $ext = 'ini';
    } elsif ( $module =~ /::Storable$/ )
    {
        $ext = 'store';
    } elsif ( $module =~ /::ConfigFile$/ )
    {
        $^W  = 0;        # To shut up Parse::Token about redefinitons
        $ext = 'conf';
    } else
    {
        $ext = 'conf';
    }

    ### Create a dummy psg.$ext file, if it does not exists
    if ( !-f "etc/psg.${ext}" )
    {
        open( FP, "> etc/psg.${ext}" );
        close FP;
    }

    ### Create the t/psgconf_modules.test file
    if ( open( FP, '> t/psgconf_modules.test' ) )
    {
        printf FP "DataStore	$module	config_file=etc/psg.${ext}\n";
        close FP;
    } else
    {
        ok( 0, 1, "Could not create t/psgconf_modules.test file" );
    }

    eval "use $module;";

    my ($psgconf) =
      PSGConf->new( %t::psgconf_options,
        modules_file => './t/psgconf_modules.test' );
    my ($ds) = $module->new( $psgconf, ("config_file=etc/psg.${ext}") );

    ok( defined $ds, 1, "$module->new failed" );

    eval { $ds->read_config($psgconf); };

    ok( $?, 0, "$module->read_config failed" );

    unlink("etc/psg.${ext}")
      if ( !-s "etc/psg.${ext}" );
}

unlink('t/psgconf_modules.test')
  if ( -f 't/psgconf_modules.test' );
