
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  06-pod.t - test script for testing documentation syntax.
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

use Test;
use File::Path;
use Pod::Checker;

use lib './t';
use t;

File::Find::find( { wanted => \&t::wanted }, 'lib' );

# Get the list of directives
@lines = `blib/script/psgconf -D -m ./etc/psgconf_modules`;
map {
    chomp;
    @a      = split;
    $module = 'lib/' . $a[2] . '.pm';
    $module =~ s|::|/|g;
    push( @{ $directive->{$module} }, $a[0] );
    $cnt++;
} @lines;

# Get the list of policies
@lines = `blib/script/psgconf -P -m ./etc/psgconf_modules`;
map {
    chomp;
    @a      = split;
    $module = 'lib/' . $a[1] . '.pm';
    $module =~ s|::|/|g;
    push( @{ $policy->{$module} }, $a[0] );
    $cnt++;
} @lines;

plan tests => scalar @t::found * 5 + $cnt + 195;

%options = ( '-warnings' => 2 );

foreach $module (@t::found)
{

    ### If we have Pod::Checker < 1.4, then it is broken with errors like this:
    ###     ERROR: unresolved internal link 'psgconf-intro' at
    if ( $Pod::Checker::VERSION < 1.4 )
    {
        skip( "Pod::Checker ($Pod::Checker::VERSION) has bugs, skipping", 1 );
    } else
    {
        $checker = new Pod::Checker %options;
        $res     = $checker->parse_from_file($module);
        ok( $res, 0, "$module failed Pod Check" );

        # Check to see if the standard sections exists
        # NAME, SYNOPSIS, DESCRIPTION and SEE ALSO
        foreach $name ( 'NAME', 'SYNOPSIS', 'DESCRIPTION', 'SEE ALSO' )
        {
            $res = $checker->name($name);
            ok( $res, $name, "$module does not have =head1 $name" );
        }

        # Now check for the specific methods defined for
        # each type of perl module
        if ( $module =~ m|^lib/PSGConf/Action/[^/]*\.pm$| )
        {
            foreach $name ( 'check', 'diff', 'do' )
            {
                $res = $checker->node( $name . '()' );
                ok( $res, $name . '()', "$module does not have =item $name()" );
            }
        } elsif ( $module =~ m|^lib/PSGConf/Action/GenerateFile/[^/]*\.pm$| )
        {
            $res = $checker->node('generate()');
            ok( $res, 'generate()', "$module does not have =item generate()" );
        } elsif ( $module =~ m|^lib/PSGConf/Action/PackageManager/[^/]*\.pm$| )
        {
        } elsif ( $module =~ m|^lib/PSGConf/Control/.*\.pm$| )
        {
            foreach $name ( 'new', 'decide' )
            {
                $res = $checker->node( $name . '()' );
                ok( $res, $name . '()', "$module does not have =item $name()" );
            }
            foreach $name ( @{ $directive->{$module} } )
            {
                $res = $checker->node($name);
                ok( $res, $name, "$module does not have =item $name" );
            }
            foreach $name ( @{ $policy->{$module} } )
            {
                $res = $checker->node($name);
                ok( $res, $name, "$module does not have =item $name" );
            }
        } elsif ( $module =~ m|^lib/PSGConf/Data/.*\.pm$| )
        {
        } elsif ( $module =~ m|^lib/PSGConf/DataStore/.*\.pm$| )
        {
            foreach $name ( 'new', 'read_config' )
            {
                $res = $checker->node( $name . '()' );
                ok( $res, $name . '()', "$module does not have =item $name()" );
            }
        }

        $checker = undef;
    }

    ### FIXME:Need to verify that each module has the following:
    ###		Every method defined has a POD item as well.
    ###
    ### 		Control:  Every action defined has a corresponding POD item.
}

rmtree( $ENV{'PSGCONF_TMPDIR'} ) if ( -d $ENV{'PSGCONF_TMPDIR'} );
